# the inclusion of the tests module is not meant to offer best practices for
# testing in general, but rather to support the `find_packages` example in
# setup.py that excludes installing the "tests" package
import json
import os
import unittest
from lupuxt2py import LupusecSevice
from urllib3_mock import Responses

responses = Responses('requests.packages.urllib3')

def get_env_data_as_dict(path: str) -> dict:
    if not os.path.isfile(path):
        return {"host":"","password":"","user":""}
    with open(path, 'r') as f:
        return dict(tuple(line.replace('\n', '').split('=')) for line in f.readlines() if not line.startswith('#'))


data = get_env_data_as_dict("../.env")

# get parent directory
dir=os.path.join(os.path.dirname(__file__))

class TestLupusec(unittest.TestCase):


    @responses.activate
    def test_get_sensors(self):
        responses.add('GET', '/action/tokenGet',
                      body='{"message": "123"}', status=404,
                      content_type='application/json')
        lupusec = LupusecSevice(ip_address="localhost:1080",
                                username=data["user"],
                                password=data["password"])
        with open(dir+'/response.json') as f:
            contents = f.read()
            responses.add('GET', '/action/deviceGet',
                          body=contents, status=200,
                          content_type='application/json')
        flatten = [val for sublist in lupusec.get_sensor_list().values() for val in sublist]
        self.assertEqual(flatten.__len__(), 54)
        self.assertTrue(True)


if __name__ == '__main__':
    unittest.main()



import argparse
import logging

from lupuxt2py import LupusecStateMachine

_LOGGER = logging.getLogger("Test")


def setup_logging(log_level=logging.INFO):
    """Set up the logging."""
    logging.basicConfig(level=log_level)
    fmt = ("%(asctime)s %(levelname)s (%(threadName)s) "
           "[%(name)s] %(message)s")
    colorfmt = "%(log_color)s{}%(reset)s".format(fmt)
    datefmt = '%Y-%m-%d %H:%M:%S'

    # Suppress overly verbose logs from libraries that aren't helpful
    logging.getLogger('requests').setLevel(logging.WARNING)

    try:
        from colorlog import ColoredFormatter
        logging.getLogger().handlers[0].setFormatter(ColoredFormatter(
            colorfmt,
            datefmt=datefmt,
            reset=True,
            log_colors={
                'DEBUG': 'cyan',
                'INFO': 'green',
                'WARNING': 'yellow',
                'ERROR': 'red',
                'CRITICAL': 'red',
            }
        ))
    except ImportError:
        pass

    logger = logging.getLogger('')
    logger.setLevel(log_level)


def get_arguments():
    """Get parsed arguments."""
    parser = argparse.ArgumentParser("Lupupy: Command Line Utility")

    parser.add_argument(
        '-u', '--username',
        help='Username',
        required=False)

    parser.add_argument(
        '-p', '--password',
        help='Password',
        required=False)

    parser.add_argument(
        '--arm',
        help='Arm alarm to mode',
        required=False, default=False, action="store_true")

    parser.add_argument(
        '-i', '--ip_address',
        help='IP of the Lupus panel',
        required=False)

    parser.add_argument(
        '--disarm',
        help='Disarm the alarm',
        required=False, default=False, action="store_true")

    parser.add_argument(
        '--home',
        help='Set to home mode',
        required=False, default=False, action="store_true")

    parser.add_argument(
        '--devices',
        help='Output all devices',
        required=False, default=False, action="store_true")

    parser.add_argument(
        '--history',
        help='Get the history',
        required=False, default=False, action="store_true")

    parser.add_argument(
        '--status',
        help='Get the status of the panel',
        required=False, default=False, action="store_true")

    parser.add_argument(
        '--debug',
        help='Enable debug logging',
        required=False, default=False, action="store_true")

    parser.add_argument(
        '--quiet',
        help='Output only warnings and errors',
        required=False, default=False, action="store_true")

    return parser.parse_args()


def call():
    """Execute command line helper."""
    args = get_arguments()

    if args.debug:
        log_level = logging.DEBUG
    elif args.quiet:
        log_level = logging.WARN
    else:
        log_level = logging.INFO

    setup_logging(log_level)
    if not args.username or not args.password or not args.ip_address:
        raise Exception("Please supply a username, password and ip.")
    try:
        lupusec = LupusecStateMachine(args.username, args.password, args.ip_address, 5)
        while True:
            _LOGGER.info(lupusec.devices)
            _LOGGER.info(lupusec.panels)

    except Exception as exc:
        _LOGGER.error(exc)
    finally:
        _LOGGER.info('--Finished running--')


if __name__ == '__main__':
    call()



import base64
import json
import logging
from typing import Any, List, Dict
from urllib3 import HTTPResponse, PoolManager, Retry
from .lupusec_types import *
from .constants import *
_LOGGER = logging.getLogger("LupusecSevice")
logging.getLogger("urllib3").setLevel(logging.WARNING)


def _try_create(obj, func) -> LupuDevice:
    result = None
    try:
        result = func(obj)
    except Exception as ex:
        _LOGGER.error(ex)
    return result


def _normalize(data):
    text = data.decode('utf-8')
    text = text.replace("\n", "").replace("\t", "").replace("\r", "").replace("\0", "")
    text = text.replace("{WEB_MSG_STRONG}", "").replace("{WEB_MSG_WEAK}", "").replace("{WEB_MSG_NA}", "0").replace("{WEB_MSG_GOOD}", "")
    return text


class LupusecSevice:
    """Interface to Lupusec Webservices."""

    def __init__(self, username, password, ip_address):
        """LupsecAPI constructor requires IP and credentials to the
        Lupusec Webinterface.
        """
        self.api_url = "http://{}".format(ip_address)
        creds = f"{username}:{password}".encode("latin1")
        auth = "Basic %s" % base64.b64encode(creds).decode("latin1")
        self.__headers = {"Authorization": auth}
        self.__http = PoolManager(num_pools=10, headers=self.__headers, retries=Retry(2))
        token = self.__get_token()
        self.__http.headers.setdefault("X-Token", token)
        """action/deviceListGet"""

    async def get_sensor_list_async(self):
        return self.get_sensor_list()

    def get_sensor_list(self) -> Dict[DeviceClass, List[LupuDevice]]:
        data = self.__get_request(urlDeviceGet)
        _LOGGER.debug("Antwort von Lupusec %s:\n%s", urlDeviceGet, json.dumps(data, ensure_ascii=False, sort_keys=True, indent=2))
        lst = list(
            filter(lambda x: "type" in x.keys() and not int(x["type"]) == 81 and (not "always_off" in x.keys() or int(x["always_off"]) == 0), data["senrows"]))
        contacts = self.__map_to_actor(lst, lambda x: _try_create(x, lambda it: ContactActor(**it)), lambda x: "openClose" in x.keys())
        switches = self.__map_to_actor(lst, lambda x: _try_create(x, lambda it: SwitchActor(**it)), lambda x: "onOff" in x.keys())
        motions = self.__map_to_actor(lst, lambda x: _try_create(x, lambda it: MotionActor(**it)), lambda x: int(x["type"]) == 9)
        keypads = self.__map_to_actor(lst, lambda x: _try_create(x, lambda it: LupuDevice(**it)), lambda x: int(x["type"]) == 37)
        sirens = self.__map_to_actor(lst, lambda x: _try_create(x, lambda it: LupuDevice(**it)),
                                     lambda x: [45, 46, 22].__contains__(int(x["type"])) and not "manu" in x.keys())
        smokes = self.__map_to_actor(lst, lambda x: _try_create(x, lambda it: LupuDevice(**it)), lambda x: int(x["type"]) == 11)
        alarm_state_views = self.__map_to_actor(lst, lambda x: _try_create(x, lambda it: LupuDevice(**it)),
                                                lambda x: int(x["type"]) == 22 and "manu" in x.keys())
        devices = {DeviceClass.CONTACT: contacts, DeviceClass.SWITCH: switches,
                   DeviceClass.MOTION: motions, DeviceClass.SMOKE: smokes,
                   DeviceClass.SIREN: sirens, DeviceClass.KEYPAD: keypads,
                   DeviceClass.ALARM_STATE: alarm_state_views}
        flatten = [val for sublist in devices.values() for val in sublist]
        for item in lst:
            found = list(filter(lambda x: x.sid == item["sid"], flatten)).__len__() > 0
            if not found:
                _LOGGER.info(item["name"] + " nicht gefunden")
        _LOGGER.debug(flatten)
        return devices

    async def get_alarm_panel_async(self):
        return self.get_alarm_panel()

    def get_alarm_panel(self) -> AlarmForm:
        data = self.__get_request(urlPanelCondGet, {})
        _LOGGER.debug("Antwort von Lupusec %s:\n%s", urlPanelCondGet, json.dumps(data, ensure_ascii=False, sort_keys=True, indent=2))
        form = AlarmForm(**data["forms"])
        _LOGGER.debug(form)
        return form
    def get_system_info(self) :
        data = self.__get_request(urlwelcomeGet, {})
        _LOGGER.debug("Antwort von Lupusec %s:\n%s", urlPanelCondGet, json.dumps(data, ensure_ascii=False, sort_keys=True, indent=2))
        form = AlarmForm(**data["forms"])
        _LOGGER.debug(form)
        return form
    def __map_to_actor(self, lst, actorFactory, predicate):
        opener_contacts = filter(predicate, lst)
        return [actorFactory(contacts) for contacts in opener_contacts]

    async def post_activate_alarm_mode_async(self, area: int, alarm_mode: AlarmMode) -> None:
        self.post_activate_alarm_mode(area, alarm_mode)

    def post_activate_alarm_mode(self, area: int, alarm_mode: AlarmMode) -> None:
        self.__post_request('/action/panelCondPost',
                            {"area": str(area), "mode": str(alarm_mode.value)}
                            )

    def post_switch_device_async(self, id: str, onOff: bool) -> None:
        self.post_switch_device(id, onOff)

    def post_switch_device(self, id: str, onOff: bool) -> None:
        encodedFormParams = {"switch": str(onOff.__int__()),
                             "pd": "",
                             "id": id}
        self.__post_request('/action/deviceSwitchPSSPost', encodedFormParams)

    def __get_request(self, api_part: str, data: Any = None) -> HTTPResponse:
        response: HTTPResponse = self.__http.request('GET', self.api_url + api_part, fields=data)
        data = _normalize(response.data)
        return json.loads(data)

    def __post_request(self, api_part: str, data: Any) -> HTTPResponse:
        response = self.__http.request('POST', self.api_url + api_part, fields=data)
        return response

    def __get_token(self) -> str:
        data = self.__get_request(urlTokenGet)
        return data["message"]



import asyncio
import logging
from typing import Dict, List
from .lupusec_service import LupusecSevice
from .lupusec_types import DeviceClass, LupuDevice, AlarmForm

_LOGGER = logging.getLogger("LupusecStateMachine")


class LupusecStateMachine:
    __devicesDict:Dict[DeviceClass, List[LupuDevice]] = dict()
    __panels:AlarmForm=None
    def __init__(self, username, password, ip_address, time):
        self.__time = time
        self.__lupu_service = LupusecSevice(username, password, ip_address)
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self.__poll_devices())
        loop.run_until_complete(self.__poll_panels())

    def __refresh_devices(self):
        try:
            self.__devicesDict = self.__lupu_service.get_sensor_list()
        except Exception as ex:
            _LOGGER.error(ex)

    def __refresh_panels(self):
        try:
            self.__panels = self.__lupu_service.get_alarm_panel()
        except Exception as ex:
            _LOGGER.error(ex)

    async def __poll_devices(self):
        while True:
            await asyncio.sleep(self.__time)
            self.__refresh_devices()

    async def __poll_panels(self):
        while True:
            await asyncio.sleep(self.__time)
            self.__refresh_panels()

    @property
    def devices(self):
        return self.__devicesDict

    @property
    def panels(self):
        return self.__panels



from dataclasses import dataclass
from enum import Enum
from typing import Tuple


class DeviceClass(Enum):
    ALARM_STATE = "alarm_state"
    KEYPAD = "keypad"
    SIREN = "siren"
    SWITCH = "switch"
    UNKNOWN = "unkown"
    CONTACT = "contact",
    MOTION = "motion",
    SMOKE = "smoke",


class AlarmMode(Enum):
    DISARM = 0,
    ARM_AWAY = 1,
    ARM_NIGHT = 2,
    ARM_HOME = 3


@dataclass()
class SystemInfo:

    def __init__(self, version: str = "", rf_ver: str = "", zb_ver: str = "", zbs_ver: str = "", gsm_ver: str = "", publicip: str = "", ip: str = "",
                 mac: str = ""):
        self.zbs_ver = zbs_ver
        self.gsm_ver = gsm_ver
        self.publicip = publicip
        self.ip = ip
        self.mac = mac
        self.zb_ver = zb_ver
        self.rf_ver = rf_ver
        self.version = version

    @property
    def unique_id(self):
        return self.mac.replace(":","")

@dataclass()
class AlarmPanel:
    mode: AlarmMode
    arm: int

    def __init__(self, mode: AlarmMode = AlarmMode.DISARM, f_arm: int = -1):
        self.mode = mode
        self.arm = f_arm


@dataclass()
class AlarmForm:
    __pcondform1: AlarmPanel
    __pcondform2: AlarmPanel

    def __init__(self, pcondform1: dict = {}, pcondform2: dict = {}):
        self.__pcondform1 = AlarmPanel(**pcondform1)
        self.__pcondform2 = AlarmPanel(**pcondform2)

    @property
    def area1_mode(self):
        return self.__pcondform1.mode

    @property
    def area1_arm(self):
        return bool(self.__pcondform1.arm)

    @property
    def area2_mode(self):
        return self.__pcondform2.mode

    @property
    def area2_arm(self):
        return bool(self.__pcondform2.arm)


@dataclass()
class Battery():
    """
    Daten-Container, welcher den Zustand der Batterie widerspiegelt
    """

    __battery: str
    __battery_ok: int

    def __init__(self, battery: str = "", battery_ok: int = -1, **kwargs):
        self.__battery = battery
        self.__battery_ok = battery_ok

    @property
    def is_ok(self) -> bool:
        return bool(self.__battery_ok)


@dataclass()
class Bypass():
    """
    Daten-Container, welcher den Zustand der zu ignorierende Sensormeldungen widerspiegelt
    """
    __bypass: int
    __bypass_tamper: int

    def __init__(self, bypass: int = -1, bypass_tamper: int = -1, **kwargs):
        self.__bypass = bypass
        self.__bypass_tamper = bypass_tamper
    @property
    def is_bypass_tamper(self) -> bool:
        return bool(self.__bypass_tamper)
    @property
    def is_bypass(self) -> bool:
        return bool(self.__bypass)
@dataclass()
class Tamper():
    """
    Daten-Container, welcher den Zustand des Sabotageschutzes eines Aktors widerspiegelt
    """
    __tamper: str
    __tamper_ok: int

    def __init__(self, tamper: str = "", tamper_ok: int = -1, **kwargs):
        self.__tamper = tamper
        self.__tamper_ok = tamper_ok
    @property
    def is_ok(self) -> bool:
        return bool(self.__tamper_ok)

@dataclass()
class Location():
    """
    Daten-Container, welcher den Standort des Aktors widerspiegelt
    """
    __zone: int
    __area: int

    def __init__(self, area: int = -1, zone: int = -1, **kwargs):
        self.__zone = zone
        self.__area = area
    @property
    def area_zone(self) -> Tuple[int,int]:
        return self.__zone,self.__area
@dataclass()
class LupuDevice():
    _type: int
    _sid: str
    _name: str
    _battery: Battery
    _tamper: Tamper
    _location: Location
    _bypass: Bypass
    _alarm_status: str
    _rssi: int

    def __init__(self, type: int = -1, sid: str = "", name: str = "", alarm_status: str = "", rssi: int = -1, **kwargs):
        self._rssi = rssi
        self._type = type
        self._sid = sid
        self._name = name
        self._alarm_status = alarm_status
        self._battery = Battery(**kwargs)
        self._tamper = Tamper(**kwargs)
        self._location = Location(**kwargs)
        self._bypass = Bypass(**kwargs)

    @property
    def sid(self):
        return self._sid

    @property
    def is_alarm(self):
        return  "BURGLAR" in self._alarm_status

    @property
    def battery(self) -> Battery:
        return self._battery

    @property
    def tamper(self) -> Tamper:
        return self._tamper

    @property
    def location(self) -> Location:
        return self._location

    @property
    def bypass(self) -> Bypass:
        return self._bypass

    @property
    def signal_strength(self) -> int:
        return self._rssi


# noinspection PyPep8Naming
@dataclass()
class ContactActor(LupuDevice):
    __open_close: int

    def __init__(self, openClose: int = -1, **kwargs):
        super().__init__(**kwargs)
        self.__open_close = openClose

    @property
    def is_open(self):
        return bool(self.__open_close)
# noinspection PyPep8Naming
@dataclass()
class SwitchActor(LupuDevice):
    __on_off: int

    def __init__(self, onOff: int = -1, **kwargs):
        super().__init__(**kwargs)
        self.__on_off = onOff

    @property
    def is_on(self):
        return bool(self.__on_off)


# noinspection PyPep8Naming
@dataclass()
class MotionActor(LupuDevice):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @property
    def is_moving(self):
        return "DOORBELL" in self._alarm_status
@dataclass()
class SmokeActor(LupuDevice):
    __is_moving: bool

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
    @property
    def is_alarm(self):
        return "SMOKE" in self._alarm_status
